function KFSMMC_Toy

  Reg=0; % 0 = L1, 1 = L2
  Kfolds = 5;
  lambda = 1;
    
  [K, Y, NUMkernels] = createToyDataAndComputeKernels (1, 1);
        
  indices = crossvalind('Kfold',size(Y,1),Kfolds);

%   aa = randperm(size(Y,1));
%   X(:,:) = X(:,aa);
%   Y(:,:) = Y(aa,:);

  cvs = cell(Kfolds,1);
  for cv = 1:Kfolds
     cvs{cv} = find(indices == cv)';
  end
  
  cp = classperf (cellstr(num2str(Y)));
  %Test 
  accuracies = zeros(1,Kfolds);
  allds = zeros(Kfolds,NUMkernels);
  for cvIdx = 1:Kfolds
      tstIdx = cvs{cvIdx};
      trnIdx = setdiff([cvs{:}], tstIdx);

      Ytrn = Y(trnIdx,:);
      Ytst = Y(tstIdx,:);

      if (Kfolds == 1)
          Ytrn = Ytst;
          trnIdx = tstIdx;
      end
      
      global Kpre;

      % Training
      parms=initparms(NUMkernels,2,Reg,lambda);

      Kpre = K (trnIdx,trnIdx,:);

      model=Optimize([1:size(Kpre,1)],Ytrn,parms);

      % Testing
      Kpre = K (trnIdx,tstIdx,:);

      KK=parms.fncK(model.svind,[1:size(Kpre,2)],parms,model.d);
      predict=sign(model.b+model.alphay'*KK)';
      accuracy=100*mean(predict==Ytst);
      fprintf('Classification accuracy on the %d-th test set = %6.2f%%\n', cvIdx, accuracy); 
      allds(cvIdx,:) = model.d';
      accuracies(cvIdx) = accuracy;
      classperf (cp, cellstr(num2str(predict)), tstIdx);
      [XX,YY,TT,AUC] = perfcurve(Ytst,(model.b+model.alphay'*KK)',1);
      allAUCs(cvIdx) = AUC;
  end
  fprintf('Classification mean accuracy (10-fold) = %6.2f%%\n', mean(accuracies)); 
  fprintf('SEN = %6.2f, SPE = %6.2f, AUC = %6.2f\n', cp.Sensitivity, cp.Specificity, mean(allAUCs)); 
  fprintf('%f, ', allds);
  fprintf('\n');
 
  Ds = [mean(mean(allds(:,1:50))), mean(mean(allds(:,51:100))), mean(mean(allds(:,101:150))), mean(mean(allds(:,151:200))), mean(mean(allds(:,201:250)))];
  fprintf('%f, ', Ds);
  fprintf('\n');
  %pause;
  %save ('Results_KFSMMC.mat');

end

function D=precomp(TRNfeatures,TSTfeatures)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk, D(:,:,k)=(repmat(TRNfeatures(k,:)',1,NUMtest)-repmat(TSTfeatures(k,:),NUMtrain,1)).^2; end;
end

function D=HIKprecomp(TRNfeatures,TSTfeatures,beta)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk, D(:,:,k)=bsxfun(@min,abs(repmat(TRNfeatures(k,:)',1,NUMtest)).^beta, abs(repmat(TSTfeatures(k,:),NUMtrain,1)).^beta); end;        
end

function D=Chi2precomp(TRNfeatures,TSTfeatures)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk
  %    A=repmat(TRNfeatures(k,:)',1,NUMtest);
  %    B=repmat(TSTfeatures(k,:),NUMtrain,1);
%       D(:,:,k)=pdist2(TRNfeatures(k,:)',TSTfeatures(k,:)','chisq');
      D(:,:,k)=pdist2(TRNfeatures(k,:)',TSTfeatures(k,:)','chisq');
  end;        
end

function D=EMD2precomp(TRNfeatures,TSTfeatures)
  [NUMk,NUMtrain]=size(TRNfeatures);
  [NUMk,NUMtest]=size(TSTfeatures);
  D=zeros(NUMtrain,NUMtest,NUMk);
  for k=1:NUMk
  %    A=repmat(TRNfeatures(k,:)',1,NUMtest);
  %    B=repmat(TSTfeatures(k,:),NUMtrain,1);
%       D(:,:,k)=pdist2(TRNfeatures(k,:)',TSTfeatures(k,:)','chisq');
      D(:,:,k)=pdist2(TRNfeatures(k,:)',TSTfeatures(k,:)','emd');
  end;        
end

function parms=initparms(NUMk,TYPEker,TYPEreg,lambda)
  % Kernel parameters
  switch (TYPEker),
    case 0, parms.KERname='Sum of RBF kernels across features';
            parms.gamma=ones(NUMk,1);
	    parms.fncK=@KSumRBF;
	    parms.fncKdash=@KdashSumRBF;
	    parms.fncKdashint=@KdashSumRBFintermediate;
    case 1, parms.KERname='Product of RBF kernels across features';
            parms.gamma=0.001;
	    parms.fncK=@KProdRBF;
	    parms.fncKdash=@KdashProdRBF;
	    parms.fncKdashint=@KdashProdRBFintermediate;
    case 2, parms.KERname='Sum of precomputed kernels';
	    parms.fncK=@KSumPrecomp;
	    parms.fncKdash=@KdashSumPrecomp;
	    parms.fncKdashint=@KdashSumPrecompintermediate;
    case 3, parms.KERname='Product of exponential kernels of precomputed disance matrices';
            parms.gamma=0.5;
	    parms.fncK=@KProdExpPrecomp;
	    parms.fncKdash=@KdashProdExpPrecomp;
	    parms.fncKdashint=@KdashProdExpPrecompintermediate;
    otherwise, fprintf('Unknown kernel type.\n'); keyboard; 
  end;
      
  % Regularization parameters
  switch (TYPEreg),
    case 0, parms.REGname='l1';          % L1 Regularization
            parms.sigma=ones(NUMk,1)*lambda;
	    parms.fncR=@Rl1;
	    parms.fncRdash=@Rdashl1;
    case 1, parms.REGname='l2';          % L2 Regularization
            parms.mud=ones(NUMk,1);
	    parms.covd=1e-1*eye(NUMk);
	    parms.invcovd=inv(parms.covd);
	    parms.fncR=@Rl2;
	    parms.fncRdash=@Rdashl2;
    otherwise, fprintf('Unknown regularisation type.\n'); keyboard; 
  end;
    
  % Standard SVM parameters
  parms.TYPEprob=0;           % 0 = C-SVC, 3 = EPS-SVR (try others at your own risk)
  parms.C=10;                 % Misclassification penalty for SVC/SVR ('-c' in libSVM)
  
  % Gradient descent parameters
  parms.initd=rand(NUMk,1);   % Starting point for gradient descent
  parms.TYPEsolver='MAT';     % Use Matlab's quadprog as an SVM solver.
  parms.TYPEstep=1;           % 0 = Armijo, 1 = Variant Armijo, 2 = Hessian (not yet implemented)
  parms.MAXITER=100;          % Maximum number of gradient descent iterations
  parms.MAXEVAL=400;          % Maximum number of SVM evaluations
  parms.MAXSUBEVAL=20;        % Maximum number of SVM evaluations in any line search
  parms.SIGMATOL=0.3;         % Needed by Armijo, variant Armijo for line search
  parms.BETAUP=2.1;           % Needed by Armijo, variant Armijo for line search
  parms.BETADN=0.3;           % Needed by variant Armijo for line search
  parms.BOOLverbose=true;     % Print debug information at each iteration
  parms.SQRBETAUP=parms.BETAUP*parms.BETAUP;
end

function r=Rl1(parms,d), r=parms.sigma'*d; end
function rdash=Rdashl1(parms,d), rdash=parms.sigma; end

function r=Rl2(parms,d), delta=parms.mud-d;r=0.5*delta'*parms.invcovd*delta; end
function rdash=Rdashl2(parms,d), rdash=parms.invcovd*(d-parms.mud); end

function K=KSumRBF(TRNfeatures,TSTfeatures,parms,d)
  [NUMdims,NUMtrn]=size(TRNfeatures);
  [NUMdims,NUMtst]=size(TSTfeatures);
  if (NUMdims~=length(d)), fprintf('NUMdims not equal to NUMk\n'); keyboard; end;
  
  nzind=find(d>1e-4);
  K=zeros(NUMtrn,NUMtst);
  if (~isempty(nzind)),
    for i=1:length(nzind),
      k=nzind(i);
      Dk=(repmat(TRNfeatures(k,:)',1,NUMtst)-repmat(TSTfeatures(k,:),NUMtrn,1)).^2;
      K=K+d(k)*exp(-parms.gamma(k)*Dk);
    end;
  end;
end

function Kdashint=KdashSumRBFintermediate(TRNfeatures,TSTfeatures,parms,d)
  Kdashint=[];
end

function Kdash=KdashSumRBF(TRNfeatures,TSTfeatures,parms,d,k,Kdashint)
  [NUMdims,NUMtrn]=size(TRNfeatures);
  [NUMdims,NUMtst]=size(TSTfeatures);

  Kdash=exp(-parms.gamma(k)*(repmat(TRNfeatures(k,:)',1,NUMtst)-repmat(TSTfeatures(k,:),NUMtrn,1)).^2);
end

function K=KProdRBF(TRNfeatures,TSTfeatures,parms,d)
  [NUMdims,NUMtrn]=size(TRNfeatures);
  [NUMdims,NUMtst]=size(TSTfeatures);
  if (NUMdims~=length(d)), fprintf('NUMdims not equal to NUMk\n'); keyboard; end;
  
  nzind=find(d>1e-4);
  K=zeros(NUMtrn,NUMtst);
  if (~isempty(nzind)),
    for i=1:length(nzind),
      k=nzind(i);
      K=K+d(k)*(repmat(TRNfeatures(k,:)',1,NUMtst)-repmat(TSTfeatures(k,:),NUMtrn,1)).^2;
    end;
  end;
  K=exp(-parms.gamma*K);
end

function Kdashint=KdashProdRBFintermediate(TRNfeatures,TSTfeatures,parms,d)
  Kdashint=-parms.gamma*parms.fncK(TRNfeatures,TSTfeatures,parms,d);
end

function Kdash=KdashProdRBF(TRNfeatures,TSTfeatures,parms,d,k,Kdashint)
  [NUMdims,NUMtrn]=size(TRNfeatures);
  [NUMdims,NUMtst]=size(TSTfeatures);

  Kdash=Kdashint.*(repmat(TRNfeatures(k,:)',1,NUMtst)-repmat(TSTfeatures(k,:),NUMtrn,1)).^2;
end

function K=KSumPrecomp(TRNfeatures,TSTfeatures,parms,d)
  global Kpre;
  if (size(Kpre,3)~=length(d)), fprintf('Number of kernels does not equal number of weights.\n');keyboard; end;

  K=lincomb(Kpre(TRNfeatures,TSTfeatures,:),d);
end

function Kdashint=KdashSumPrecompintermediate(TRNfeatures,TSTfeatures,parms,d)
  Kdashint=[];
end

function Kdash=KdashSumPrecomp(TRNfeatures,TSTfeatures,parms,d,k,Kdashint)
  global Kpre;
  
  Kdash=Kpre(TRNfeatures,TSTfeatures,k);
end

function K=KProdExpPrecomp(TRNfeatures,TSTfeatures,parms,d)
  global Kpre;
  if (size(Kpre,3)~=length(d)), fprintf('Number of kernels does not equal number of weights.\n');keyboard; end;

  K=exp(-parms.gamma*lincomb(Kpre(TRNfeatures,TSTfeatures,:),d));
end

function Kdashint=KdashProdExpPrecompintermediate(TRNfeatures,TSTfeatures,parms,d)
  Kdashint=-parms.gamma*parms.fncK(TRNfeatures,TSTfeatures,parms,d);
end

function Kdash=KdashProdExpPrecomp(TRNfeatures,TSTfeatures,parms,d,k,Kdashint)
  global Kpre;
  
  Kdash=Kdashint.*Kpre(TRNfeatures,TSTfeatures,k);
end

function K=lincomb(base,d)
  nzind=find(d>1e-4);
  K=zeros(size(base,1),size(base,2));
  if (~isempty(nzind)), for k=1:length(nzind), K=K+d(nzind(k))*base(:,:,nzind(k)); end; end;
end
